/* Fesslix - Stochastic Analysis
 * Copyright (C) 2010-2017 Wolfgang Betz
 *
 * Fesslix is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fesslix is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fesslix.  If not, see <http://www.gnu.org/licenses/>. 
 */

#ifndef fesslix_stat_H
#define fesslix_stat_H

#include "flxlibs_interface.h"

#include "flxStatBox.h"


class flxStatBox_container {
  private:
    #if FLX_DEBUG
      /**
      * @brief number of instances of this class - only one class may exist
      */
      static int Cinst;
    #endif
    std::map<std::string, FlxStatBox*> box;
  public:
    flxStatBox_container();
    ~flxStatBox_container();
    
    void insert (const std::string name, const tuint N, const tuint M);
    FlxStatBox& get ( const std::string& name );
};



// -------------------------- stat_box_new ----------------------------------------------------------------------

class FlxObjStat_Box_new : public FlxObjBase {
  private:
    FlxString* fname;
    FlxFunction* N;
    FlxFunction* M;
    void task();
  public:
    FlxObjStat_Box_new( const bool dolog, FlxString* fname, FlxFunction* N, FlxFunction* M) : FlxObjBase(dolog), fname(fname), N(N), M(M) {}
    ~FlxObjStat_Box_new();
};

/**
* @brief object read class: FlxObjStat_Box_new
*/
class FlxObjReadStat_Box_new : public FlxObjReadBase {
  public:
    FlxObjReadStat_Box_new() : FlxObjReadBase(true) {}
    FlxObjBase* read ();
};


// -------------------------- stat_box_add ----------------------------------------------------------------------

class FlxObjStat_Box_add : public FlxObjBase {
  private:
    FlxString* fname;
    FlxString* vecName;
    void task();
  public:
    FlxObjStat_Box_add( const bool dolog, FlxString* fname, FlxString* vecName) : FlxObjBase(dolog), fname(fname), vecName(vecName) {}
    ~FlxObjStat_Box_add();
};

/**
* @brief object read class: FlxObjStat_Box_add
*/
class FlxObjReadStat_Box_add : public FlxObjReadBase {
  public:
    FlxObjReadStat_Box_add() : FlxObjReadBase(true) {}
    FlxObjBase* read ();
};

// -------------------------- stat_box_vec_mean ----------------------------------------------------------------------

class FlxObjStat_Box_vec_mean : public FlxObjBase {
  private:
    FlxString* fname;
    FlxString* vecName;
    void task();
  public:
    FlxObjStat_Box_vec_mean( const bool dolog, FlxString* fname, FlxString* vecName) : FlxObjBase(dolog), fname(fname), vecName(vecName) {}
    ~FlxObjStat_Box_vec_mean();
};

/**
* @brief object read class: FlxObjStat_Box_add
*/
class FlxObjReadStat_Box_vec_mean : public FlxObjReadBase {
  public:
    FlxObjReadStat_Box_vec_mean() : FlxObjReadBase(true) {}
    FlxObjBase* read ();
};

// -------------------------- Run true model ----------------------------------------------------------------------

class FlxObjStat_Box_confi : public FlxObjOutputBase {
  private:
    FlxString* fname;
    FlxMtxConstFun* clf;
    int fixW;
    const std::string userColName;
    
    void task();
  public:
    FlxObjStat_Box_confi( const bool dolog, const std::string& ostreamV, FlxString* fname, FlxMtxConstFun* clf, int fixW, const std::string& userColName) : FlxObjOutputBase(dolog,ostreamV), fname(fname), clf(clf), fixW(fixW), userColName(userColName) {}
    ~FlxObjStat_Box_confi() { delete fname; delete clf; }
};

/**
* @brief object read class: FlxObjStat_Box_confi
*/
class FlxObjReadStat_Box_confi : public FlxObjReadOutputBase {
  public:
    FlxObjReadStat_Box_confi();
    FlxObjBase* read ();
};


// -------------------------- CDF from large data files --------------------------------------------------------------------------------------------------------------------------


struct flx_cdf_entry {
  float value;
  tuint occ;
};

class flx_cdf {
  private:
    size_t N;			// Note: N >= list.size() 
    std::vector<flx_cdf_entry> list;
  public:
    flx_cdf(const size_t Nreserve);
    /**
    * @brief appends a sorted vector to the data-CDF
    */
    void append(flxVec &dV);
    void plot(std::ostream& sout, const bool checkTOL = false, const int prec=-1, int fixW=-1);
    void save(const std::string& fileS);
    void load(const std::string& fileS);
};

class flx_cdf_container {
  private:
    std::map<std::string,flx_cdf*> box;
  public:
    void insert(const std::string& name, const size_t Nreserve);
    flx_cdf& get(const std::string& name);
};



// --------------------------------------------------------------

class FlxObjStat_CDF_new : public FlxObjBase {
  private:
    FlxString* fname;
    FlxFunction* Nreservef;

    void task();
  public:
    FlxObjStat_CDF_new( const bool dolog, FlxString* fname, FlxFunction* Nreservef) : FlxObjBase(dolog), fname(fname), Nreservef(Nreservef) {}
    ~FlxObjStat_CDF_new() { delete fname; delete Nreservef; }
};

class FlxObjReadStat_CDF_new : public FlxObjReadBase {
  public:
    FlxObjReadStat_CDF_new();
    FlxObjBase* read ();
};

// --------------------------------------------------------------

class FlxObjStat_CDF_append : public FlxObjBase {
  private:
    FlxString* fname;
    FlxString* istrmf;
    FlxFunction* bulkf;
    FlxFunction* transform;

    void task();
  public:
    FlxObjStat_CDF_append( const bool dolog, FlxString* fname, FlxString* istrmf, FlxFunction* bulkf, FlxFunction* transform) : FlxObjBase(dolog), fname(fname), istrmf(istrmf), bulkf(bulkf), transform(transform) {}
    ~FlxObjStat_CDF_append() { delete fname; delete istrmf; delete bulkf; delete transform; }
};

class FlxObjReadStat_CDF_append : public FlxObjReadBase {
  public:
    FlxObjReadStat_CDF_append();
    FlxObjBase* read ();
};

// --------------------------------------------------------------

class FlxObjStat_CDF_plot : public FlxObjOutputBase {
  private:
    FlxString* fname;

    void task();
  public:
    FlxObjStat_CDF_plot( const bool dolog, const std::string& ostreamV, FlxString* fname,const int prec, const int fixW) : FlxObjOutputBase(dolog,ostreamV,false,false,prec,fixW), fname(fname) {}
    ~FlxObjStat_CDF_plot() { delete fname; }
};

class FlxObjReadStat_CDF_plot : public FlxObjReadOutputBase {
  public:
    FlxObjReadStat_CDF_plot() : FlxObjReadOutputBase(true) {}
    FlxObjBase* read();
};

// --------------------------------------------------------------

class FlxObjStat_CDF_save : public FlxObjBase {
  private:
    FlxString* fname;
    FlxString* fileF;
    
    void task();
  public:
    FlxObjStat_CDF_save( const bool dolog, FlxString* fname, FlxString* fileF) : FlxObjBase(dolog), fname(fname), fileF(fileF) {}
    ~FlxObjStat_CDF_save() { delete fname; delete fileF; }
};

class FlxObjReadStat_CDF_save : public FlxObjReadBase {
  public:
    FlxObjReadStat_CDF_save() : FlxObjReadBase(true) {}
    FlxObjBase* read ();
};

// --------------------------------------------------------------

class FlxObjStat_CDF_load : public FlxObjBase {
  private:
    FlxString* fname;
    FlxString* fileF;
    
    void task();
  public:
    FlxObjStat_CDF_load( const bool dolog, FlxString* fname, FlxString* fileF) : FlxObjBase(dolog), fname(fname), fileF(fileF) {}
    ~FlxObjStat_CDF_load() { delete fname; delete fileF; }
};

class FlxObjReadStat_CDF_load : public FlxObjReadBase {
  public:
    FlxObjReadStat_CDF_load() : FlxObjReadBase(true) {}
    FlxObjBase* read ();
};





#endif // fesslix_stat_H

